

__docformat__ = "restructuredtext en"

from org.gvsig.fmap.mapcontext.layers import FLayers as JLayers

class Layers(JLayers):
  """
  Set of layers.
  
  Extends the `java FLayers`_.
  
  .. _`java FLayers` : FIXME_URL
  """

  def __len__(self):
    """
    Allow get the len of the set of layers in a python maner.
    
    :return:
      The len of the set of layers
    :returntype:
      int
    """
    return self.getLayersCount()

  def __getitem__(self, index):
    """
    Allow get items of the set of layers in a python maner.
    
    It return the layer of position *index* of the set of layers.

    :return:
      The layer requiered of the set.
    :returntype:
      Layer_
    .. _Layer : FIXME_URL
    """
    return self.getLayer(index)

  def __iter__(self):
    """
    Allow iterate over the set of layers in a python maner.
    
    Return an iterator over the set of layers.
    
    :return:
      The layers iterator
    :returntype:
      LayersIterator
    """
    return LayersIterator(self)
    

  def __call__(self):
    """
    Return the java object represented by this object.
    
    This method is for compatibility with scripting API of gvSIG 2.0
    
    :ReturnType:
      Layers
    :deprecated: 
      With gvSIG 2.1 the call this method isn't necesary
      
    """
    return self


class LayersIterator(object):
  """
  Iterator over a set of layers.
  
  This class can't be created, is used internaly by the class Layers
  when a iterator is required.
  """

  def __init__(self,layers):
    self.__layers = layers
    self.__index = -1

  def next(self):
    self.__index+=1
    if self.__index >= self.__layers.getLayersCount() :
      raise StopIteration()
    return self.__layers.getLayer(self.__index)

#
# Inject new methods in the class JLayers
#
JLayers.__call__ = Layers.__call__
JLayers.__len__ = Layers.__len__
JLayers.__getitem__ = Layers.__getitem__
JLayers.__iter__ = Layers.__iter__

