
__docformat__ = "restructuredtext en"

from org.gvsig.app.project import DefaultProject as JProject

class Project(JProject):
  """
  Represents a gvSIG project.
  
  Extends the java class DefaultProject_ .
  
  .. _DefaultProject : x
  """

  def getView(self, name=None):
    """
    Returns a view document of the project.
    
    If calling without params return the active view of the project.
    If calling with a view name and exists a view with this name
    return this, otherwise return None.
    
    :Parameters:
      name : String
        optional name of the view to retrieve if not pased return active view.
    :Returns: 
        the view document or None
    :ReturnType:
      ViewDocument_
    
    .. _ViewDocument : gvsig.view.ViewDocument-class.html
    """
    if name == None:
      try:
        activeDocument = self.getActiveDocument()
        if activeDocument == None:
            return None
        if isinstance(activeDocument, ViewDocument):
            return activeDocument
      except Exception, ex:
        raise Exception("%s" % repr(ex))
    else:
      views = self.getViews()
      if len(views) >0:
        for view in views:
          if name == view.getName():
            return view
    return None
    
  def getTable(self, name=None):
    """
    Returns a table document of the project.
    
    If calling without params return the active table of the project.
    If calling with a table name and exists a table with this name
    return this, otherwise return None.
    
    :Parameters:    
      name : String
        optional name (a string) of the table to retrieve, if not pased return active table.
    :Returns: 
        the the table document or None
    :ReturnType:
      TableDocument_

    .. _TableDocument : gvsig.table.TableDocument-class.html
    """
    if name == None:
      try:
        activeDocument = self.getActiveDocument()
        if activeDocument == None:
            return None
        if isinstance(activeDocument, TableDocument):
            return activeDocument
      except Exception, ex:
        raise Exception("%s"%repr(ex))
    else:
      tables = self.getDocuments("project.document.table")
      if len(tables) >0:
        for table in tables:
          if name == table.getName():
            return table
    return None   

  def getProjectionCode(self):
    """
    Returns Project projection code name as a string. 
    
    This value is the projects default projection that is fixed in 
    Preferences tool, View options. 
    It's used when no view projection is defined.
    
    :Returns: 
        the projection code as a string.
    :ReturnType:
      String
    """
    return self.getProjection().getFullCode()

  def __call__(self):
    """
    Return the java object represented by this object.
    
    This method is for compatibility with scripting API of gvSIG 2.0
    
    :ReturnType:
      Project
    :deprecated: 
      With gvSIG 2.1 the call this method isn't necesary
      
    """
    return self

def currentProject():
  """
  Returns current gvSIG proyect
  
  :return: 
    the proyect
  """
  application = ApplicationLocator.getManager()
  project = application.getCurrentProject()
  return Project(project)

def currentDocument(documentClass = None):
  """
  Returns the current active document.
  
  :return: 
    Active document, None
  :returntype:
    Document_
  
  .. _Document : FIXME_URL
  """
  application = ApplicationLocator.getManager()
  
  if documentClass == None:
    doc = application.getActiveDocument()
  else:
    doc = application.getActiveDocument(documentClass)
  return doc
  
def currentTable():
  """
  Returns active table document or None.
  
  :return: 
    the current table document None
  :returntype:
    TableDocument_
  .. _TableDocument : gvsig.table.TableDocument-class.html
  """  
  return currentDocument(TableDocument)
  
def currentView():
  """
  Returns the current active view document or None. 
  
  :return: 
    The current view document or None
  :returntype:
    ViewDocument_
  .. _ViewDocument : gvsig.view.ViewDocument-class.html
  """ 
  return currentDocument(ViewDocument)



    
#
# Inject new methods in the class JProject
#
JProject.getView = Project.getView
JProject.getTable = Project.getTable
JProject.getProjectionCode = Project.getProjectionCode
JProject.__call__ = Project.__call__

