
from org.gvsig.fmap.dal.feature.impl import DefaultFeatureStore

class FeatureStore(DefaultFeatureStore):
  """Represents gvsig store. It's used as Table/Layer objects store  
  (org.gvsig.fmap.dal.feature.impl.DefaultFeatureStore)
  """
  def __init__(self,store):
    WrapperToJava.__init__(self, store)
    #self.data = None
    self.fset = None
  
  def features(self, expresion = None, sortBy="", asc=True):
    """Returns layer features set (FeatureSet class), with all layer features, 
    or selected featured if there are (could be empty feature set) or None if
    gets an Exception.
    :param expresion: filter to apply to the feature set to select
        determinates features that match with expression
    :type expresion: string
    :param sortBy: name of attribute to sort
    :type sortby: string
    :param: asc: order
    :type asc: boolean
    :return: FeatureSet
    """   
    if expresion == None and sortBy =="":
      self.fset = self.getFeatureSet()         
    else:
      try:
        application = ApplicationLocator.getManager()
        datamanager =  application.getDataManager()
        query = self.createFeatureQuery()
        if sortBy != "":
            order = FeatureQueryOrder()
            order.add(sortBy, asc)
            query.setOrder(order)
        if expresion != None:
            query.setFilter(datamanager.createExpresion(expresion))
        self.fset = self.getFeatureSet(query)
      except Exception, e:
        return None
    
    return FeatureSet(self.fset)    

  def edit(self):
    """Set store in edition mode"""     
    if not self.isEditing():
        self().edit() 
     
  def append(self, *valuesList, **values):
    """
    Creates a new feature from given values and insert it in the feature 
    set. If an error occurs raises RuntimeException.
    :param values: dictionary with name property value or list named params
    :type values: dict
    """
    try:
      if len(valuesList) ==1:
        values.update(valuesList[0])
      
      if not self.isEditing():
        self.edit() 
      f = self.createNewFeature()
      
      if f == None:
        raise RuntimeError("Failed to create a new Feature")
      for k,v in values.iteritems():
        f.set(k,v)
        self.insert(f)
    except Throwable, ex:
      raise RuntimeException("Can't append values %s to layer %s (%s)" % (
        repr(values), 
        self.getName(), 
        str(ex)
        )
      )

  def updateSchema(self, schema):
    """
    Updates store FeatureType. If an error occurs raises 
    RuntimeException.
    """
    try:
      self().update(schema._javaobj)
    except Throwable, ex:
      raise RuntimeException(repr(ex))
    
  def update(self, feature):
    """
    Updates exist feature in the layer featureSet
    :param editableFeature: editableFeature
    :type editableFeature: Java editableFeature
    """
    if not self.isEditing():
      self.edit() 
    
    if not isinstance(feature, EditableFeature):
      feature = feature._javaobj
    self.fset.update(feature)
  
  def getSchema(self):
    """Returns store data attributtes"""
    return Schema(self.getDefaultFeatureType())

  def commit(self):
    """
    Finish store edition saving changes. If an error occurs raises Throwable 
    Exception.
    """
    try:
      self.finishEditing()          
    except Throwable, ex:
      self.abort()
      raise Throwable("Can't finish layer edition, cancelling changes. %s" % repr(ex))
  
  def abort(self):
    """Finish store edition withoout saving changes and disposes itself"""
    self.cancelEditing() 
    self.dispose()

  def getSelection(self):
    """Returns store features selected set"""
    return FeatureSet(self().getSelection())


