

__docformat__ = "restructuredtext en"


from org.gvsig.app.project.documents.table import TableDocument as JTableDocument


class DataTable(object):
  """
  This is the base class to TableDocument_ and LayerVectorial_
  
  .. _TableDocument : FIXME_URL
  .. _LayerVectorial : FIXME_URL
  """

  def features(self, filter = None, sortby="", asc=True):
    """
    Return a set of features
    
    Opcionalmente puede especificarse una expresion de filtrado,
    los campos por los que se quiere que se ordene y si este
    orden es ascendente o descendente.
    
    :paramters:
      filter : string
        es una expresion que deben cumplir los registros de la tabla a devolver.
      sortby : string
        es una lista de nombres de campos de la tabla por los que queremos que 
        se ordene separados por comas. 
      asc : boolean
        True para ordenar de forma ascendente, que es el valor por defecto, 
        False para ordenar descendentemete.
    
    :return:
      the set of features.
    :returntype:
      FeatureSet_
    .. _FeatureSet : FIXME_URL
    """
    return self.getFeatureStore().features(filter, sortby, asc)

  def edit(self):
    """
    Set the data table in edition mode
    """
    self.getFeatureStore().edit() 
     
  def append(self, *valuesList, **values):
    """
    Creates a new feature.
    
    Creates a new feature from given values and insert it in the feature set.
    
    This method can receibe as parameters a list of pair name=value where name
    is the name of an attribute of the feature and value the value of this attribute.
    
    Alternativement can receibe a list of values that can be asigned to the attributes
    of the feature in sequential order.

    The changes are delayed until the method commit is invoked.
    If calling the method abort the changes arent discarded.    

    """
    self.getFeatureStore().append(*valuesList, **values)
    
  def updateSchema(self, schema):
    """
    Updates the schema of the DataTable.
    """
    self.getFeatureStore().updateSchema(schema)
    
  def update(self, feature):
    """
    Updates the feature in the DataTable.
    
    If the feature exists in the DataTable this is updated with the
    values of the feature pased as parameter.

    The feature changes are delayed until the method commit is invoked.
    If calling the method abort the changes arent discarded.
    
    :Parameters:
      feature : EditableFeature_
        the feature to be updated.
    """
    self.getFeatureStore().update(feature)
  
  def getSchema(self):
    """
    Returns schema of the DataTable.
    
    The schema of the DataTable constains information about the columns of 
    the DataTable, name, type, mandatory...
    """
    return self.getFeatureStore().getSchema()

  def commit(self):
    """
    Saves all changes pending in the DataTable.
    """
    self.getFeatureStore().commit()
  
  def abort(self):
    """
    Discard all pending changes in the DataTable.
    """
    self.getFeatureStore().abort()

  def getSelection(self):
    """
    Returns features selected set
    """
    return self.getFeatureStore().getFeatureSelection()
    
  def select(self, selection):
    """
    Inserts features in the features selection set
    """
    self.getSelection().select(selection)

  def deselect(self, selection):
    """
    Remove features in the features selection set
    """
    self.getSelection().deselect(selection)

  def isSelected(feature):
    """Returns True if given feature is selected"""
    self.getSelection().isSelect(feature)


class TableDocument(JTableDocument, DataTable):
  """
  Represents gvSIG view document 

  Extends the `java TableDocument`_
  
  .. _`java TableDocument` : FIXME_URL
  """
  
  def getFeatureStore(self):
    """
    Returns the FeatureStore_ associted to this TableDocument.
    
    :return:
      The FeatureStore_ associated to the TableDocument
    :returntype:
      FeatureStore_
    .. _FeatureStore : FIXME_URL
    """
    return self.getStore()


  def __call__(self):
    """
    Return the java object represented by this object.
    
    This method is for compatibility with scripting API of gvSIG 2.0
    
    :ReturnType:
      TableDocument
    :deprecated: 
      With gvSIG 2.1 the call this method isn't necesary
      
    """
    return self
    


#
# Inject new methods in the class JTableDocument
#
JTableDocument.getFeatureStore = TableDocument.getFeatureStore
JTableDocument.__call__ = TableDocument.__call__
JTableDocument.features = DataTable.features
JTableDocument.edit = DataTable.edit
JTableDocument.append = DataTable.append
JTableDocument.updateSchema = DataTable.updateSchema
JTableDocument.update = DataTable.update
JTableDocument.getSchema = DataTable.getSchema
JTableDocument.commit = DataTable.commit
JTableDocument.abort = DataTable.abort
JTableDocument.getSelection = DataTable.getSelection
JTableDocument.select = DataTable.select
JTableDocument.deselect = DataTable.deselect
JTableDocument.isSelected = DataTable.isSelected


