
from org.gvsig.fmap.dal.feature.impl import DefaultFeatureType as JSchema

class Schema(JSchema):
  """
  Stores data properties definition. Represents gvSIG FeatureType
  """

  def append(self, name, type, size=None, default=None, precision=4):
    """Adds new property to feature properties definition. If error occurs 
    raises RuntimeError.    
    :param name: Feature property name
    :type name: String
    :param type: Feature property type
    :type name: String
    :param size: Feature property size
    :type size: int
    :param default: Feature property default value
    :return: new atribute
    """
    if not isinstance(self._javaobj, EditableFeatureType):
        self.modify()
    
    if isinstance(type, str):
      try:
        application = ApplicationLocator.getManager()
        datamanager =  application.getDataManager()
        dataTypes = application.getDataTypesManager()
        type = dataTypes.getType(type) #dataType constant value from string
      except:
        raise RuntimeError(
            "Feature Property Data type (%s) is not valid.  name=%s, type=%s, size=%s, default=%s)" % (
                type, 
                name, 
                type, 
                size, 
                default
            )
        )
    if isinstance(type, int):
      try:
        type = dataTypes.get(type)
      except:
        raise RuntimeError(
            "Data type (%s) is not valid.  name=%s, type=%s, size=%s, default=%s)" % (
                type, 
                name, 
                type, 
                size, 
                default
            )
        )
      
    attribute = self.add(name, type.getType())
 
    if size != None: 
      attribute.setSize(size)
    
    if default != None:
      attribute.setDefaultValue(default)
      
    if precision != None and type.getType() in (DataTypes.DOUBLE, DataTypes.FLOAT):
      attribute.setPrecision(precision)
    
    if type.getType() == DataTypes.GEOMETRY and self.getDefaultGeometryAttributeName()==None:
      self.setDefaultGeometryAttributeName(name)
    
    return attribute

  def __getitem__(self, name):
    return self.getAttributeDescriptor(name)

  def get(self, name, default=None):
    """Returns a feature attribute descriptor that contains information about 
    one feature attribute, such as its name, data type or precision. 
    :param name: Attribute name
    :type name: string
    :param default: Value to return if no attribute name found. 
    :return: AttributeDescriptor
    """
    x = self.getAttributeDescriptor(name)
    if x == None:
      return default
    return x
  
  def getAttrNames(self):
    """Returns a list with attributes names"""
    if not self.getAttributeDescriptors():
      return None
        
    return [attr.getName() for attr in self.getAttributeDescriptors()]
    
    
  def getCopy(self):
    """Returns a itself clone"""
    return Schema(self().getCopy())
  
  def modify(self):
    """Sets edit mode"""
    if not isinstance(self._javaobj, EditableFeatureType):
      self.featureTypeNoEditable = self._javaobj
      self._javaobj = self._javaobj.getEditable()

