

__docformat__ = "restructuredtext en"


from org.gvsig.app.project.documents.view import DefaultViewDocument as JViewDocument

class ViewDocument(JViewDocument):
  """
  
  Represents gvSIG view document 
  
  Extends the java class DefaultViewDocument_
  
  .. _DefaultViewDocument : FIXME_URL
  """

  def getLayer(self, name=None):
    """
    Return a layer of the view document.
    
    If don't pecified a layer name, return the current layer of
    the view. If has more than one active layer return None.
    If spacified a layer name and exists a layer with this name,
    return this, otherwise return None.
     
    :parameters:
      name : string
        optional name of the layer to retrieve, if not pased return active layer.
    :returns:
      the layer or None
    :returntype:
      Layer_
    .. _layer : FIXME_URL
    """
    mapContext = self.getMapContext();
    if name == None:
      activeLayers = mapContext.getLayers().getActives()
      if len(activeLayers) != 1 :
        return None
      for layer in activeLayers:
        if not isinstance(layer, FLayers):
          return layer
      return None
    layers = self.getLayers()
    for i in xrange(layers.getLayersCount()):
      layer = layers.getLayer(i)
      if layer.name == name:
        return layer
    return None
  
  def getMap(self):
    """
    Return the map definition asociated to the view.
    
    This contains the layers, projection and definition
    of the area to view.
    
    :returns:
      the map definition
    :returntype:
      MapContext_
    
    .. _MapContext : FIXME_URL
    """
    return self.getMapContext();
 
  def addLayer(self, layer):
    """
    Add a new layer to the view.
    
    Don't return any thing.
    
    :parameters:
      layer : Layer_
        the layer to add to the view
        
    .. _Layer : FIXME_URL
    """
    self.getMapContext().getLayers().addLayer(layer)
  
  def getLayers(self):
    """
    Return a iterable set of \p Layers from the view.
    
    :returns:
      a Layers object
    :returntype:
      Layers_
    
    .. _Layers : FIXME_URL
    """
    mapContext = self.getMapContext()
    return mapContext.getLayers()

  def getGraphicsLayer(self):
    """
    Return the graphics layer of the view.
    
    :returns:
      a the graphics layer
    :returntype:
       DefaultGraphicLayer_
       
    .. _DefaultGraphicLayer : FIXME_URL
    """    
    return self.getMapContext().getGraphicsLayer()

  def getProjectionCode(self):
    """
    Returns Project projection code name of the view as a string. 
    
    :return:
      the projection code as a string.
    :returntype:
      String
    """  
    return self.getProjection().getFullCode()
    
  def isProjected(self):
    """
    Returns if the coordinate system of the view is projected.
    
    :return:
      true if the coordinate system of the view is projected, otherwise return false.
    :returntype:
      boolean
    """  
    self.getProjection().isProjected()

  def __call__(self):
    """
    Return the java object represented by this object.
    
    This method is for compatibility with scripting API of gvSIG 2.0
    
    :ReturnType:
      ViewDocument
    :deprecated: 
      With gvSIG 2.1 the call this method isn't necesary
      
    """
    return self
    

   
#
# Inject new methods in the class JViewDocument
#
JViewDocument.getLayer = ViewDocument.getLayer
JViewDocument.getMap = ViewDocument.getMap
JViewDocument.addLayer = ViewDocument.addLayer
JViewDocument.getGraphicsLayer = ViewDocument.getGraphicsLayer
JViewDocument.getProjectionCode = ViewDocument.getProjectionCode
JViewDocument.isProjected = ViewDocument.isProjected
JViewDocument.__call__ = ViewDocument.__call__


